BEGIN
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'REVIEW_COVERAGE_USERS_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM REVIEW_COVERAGE_USERS_V';
  END LOOP;
  FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'REVIEW_COVERAGE_USERS_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW REVIEW_COVERAGE_USERS_V';
  END LOOP;
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'REVIEW_COVERAGE_PERMISSIONS_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM REVIEW_COVERAGE_PERMISSIONS_V';
  END LOOP;
  FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'REVIEW_COVERAGE_PERMISSIONS_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW REVIEW_COVERAGE_PERMISSIONS_V';
  END LOOP;
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'REVIEW_COVERAGE_ACCOUNTS_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM REVIEW_COVERAGE_ACCOUNTS_V';
  END LOOP;
  FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'REVIEW_COVERAGE_ACCOUNTS_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW REVIEW_COVERAGE_ACCOUNTS_V';
  END LOOP;
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'REVIEW_COVERAGE_ROLES_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM REVIEW_COVERAGE_ROLES_V';
  END LOOP;
  FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'REVIEW_COVERAGE_ROLES_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW REVIEW_COVERAGE_ROLES_V';
  END LOOP;
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'REVIEW_COVERAGE_BROLES_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM REVIEW_COVERAGE_BROLES_V';
  END LOOP;
  FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'REVIEW_COVERAGE_BROLES_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW REVIEW_COVERAGE_BROLES_V';
  END LOOP;
END;
/

CREATE VIEW review_coverage_users_v AS
  SELECT
    suser.display_name,
    suser.display_description,
    suser.risk,
    suser.department
  FROM suser
  WHERE suser.unique_user_id NOT IN (
    SELECT item.unique_user_id
    FROM review_item item
    WHERE item.unique_user_id IS NOT NULL)
    AND suser.effective = 1 AND suser.deleted = 0;

CREATE VIEW review_coverage_permissions_v AS
  SELECT
    perm.name,
    perm.description,
    perm.risk,
    app.id app_id,
    app.name app_name
  FROM spermission perm
    JOIN data_source app ON app.id = perm.application
  WHERE perm.unique_permission_id NOT IN (
    SELECT item.unique_permission_id
    FROM review_item item
    WHERE item.unique_permission_id IS NOT NULL)
    AND perm.effective = 1 AND perm.deleted = 0 AND perm.exclude_from_catalog = 0;

CREATE VIEW review_coverage_accounts_v AS
  SELECT 
    acc.name,
    acc.description,
    acc.risk,
    app.id app_id,
    app.name app_name
  FROM saccount acc
    JOIN data_source app ON app.id = acc.application
  WHERE acc.unique_account_id NOT IN (
    SELECT item.unique_account_id
    FROM review_item item
    WHERE item.unique_account_id IS NOT NULL)
    AND acc.effective = 1 AND acc.deleted = 0 AND acc.virtual = 0;

CREATE VIEW review_coverage_roles_v AS
  SELECT 
    role.name,
    role.description,
    role.risk,
    role_owners.users user_owners,
    role_owners.groups group_owners
  FROM role_policy role
    JOIN (SELECT 
        role_policy.id,
        suser.owners users,
        sgroup.owners groups
      FROM role_policy
        LEFT JOIN (SELECT 
            pol_owner.role_policy, 
            LISTAGG(TO_CHAR(suser.display_name), ' | ') WITHIN GROUP (ORDER BY suser.display_name) owners
          FROM policy_owner pol_owner
            JOIN suser ON pol_owner.unique_owner_id = suser.unique_user_id AND pol_owner.is_user = 1 AND suser.effective = 1
          WHERE pol_owner.role_policy IS NOT NULL
          GROUP BY pol_owner.role_policy) suser ON suser.role_policy = role_policy.id
        LEFT JOIN (SELECT 
            pol_owner.role_policy, 
            LISTAGG(TO_CHAR(sgroup.name), ' | ') WITHIN GROUP (ORDER BY sgroup.name) owners
          FROM policy_owner pol_owner
            JOIN sgroup ON pol_owner.unique_owner_id = sgroup.unique_group_id AND pol_owner.is_user = 0 AND sgroup.effective = 1
          WHERE pol_owner.role_policy IS NOT NULL
          GROUP BY pol_owner.role_policy) sgroup ON sgroup.role_policy = role_policy.id) role_owners ON role.id = role_owners.id
  WHERE role.id NOT IN (
    SELECT item.role_id
    FROM review_item item
    WHERE item.role_id IS NOT NULL)
    AND role.deleted = 0;

CREATE VIEW review_coverage_broles_v AS
  SELECT
    brole.name,
    brole.description,
    brole.risk,
    brole_owners.users user_owners,
    brole_owners.groups group_owners
  FROM business_role brole
    JOIN (SELECT 
        business_role.id,
        suser.owners users,
        sgroup.owners groups
      FROM business_role
        LEFT JOIN (SELECT 
            br_owner.business_role_id, 
            LISTAGG(TO_CHAR(suser.display_name), ' | ') WITHIN GROUP (ORDER BY suser.display_name) owners
          FROM business_role_owner br_owner
            JOIN suser ON br_owner.unique_owner_id = suser.unique_user_id AND br_owner.is_user = 1 AND suser.effective = 1
          WHERE br_owner.business_role_id IS NOT NULL
          GROUP BY br_owner.business_role_id) suser ON suser.business_role_id = business_role.id
        LEFT JOIN (SELECT 
            br_owner.business_role_id, 
            LISTAGG(TO_CHAR(sgroup.name), ' | ') WITHIN GROUP (ORDER BY sgroup.name) owners
          FROM business_role_owner br_owner
            JOIN sgroup ON br_owner.unique_owner_id = sgroup.unique_group_id AND br_owner.is_user = 0 AND sgroup.effective = 1
          WHERE br_owner.business_role_id IS NOT NULL
          GROUP BY br_owner.business_role_id) sgroup ON sgroup.business_role_id = business_role.id) brole_owners ON brole_owners.id = brole.id
  WHERE brole.approval_state = 'PUBLISHED' AND brole.id NOT IN ( 
      SELECT item.business_role_id
      FROM review_item item
      WHERE item.business_role_id IS NOT NULL);

GRANT SELECT ON igops.review_coverage_users_v TO ig_report_role;
CREATE PUBLIC SYNONYM review_coverage_users_v FOR igops.review_coverage_users_v;

GRANT SELECT ON igops.review_coverage_permissions_v TO ig_report_role;
CREATE PUBLIC SYNONYM review_coverage_permissions_v FOR igops.review_coverage_permissions_v;

GRANT SELECT ON igops.review_coverage_accounts_v TO ig_report_role;
CREATE PUBLIC SYNONYM review_coverage_accounts_v FOR igops.review_coverage_accounts_v;

GRANT SELECT ON igops.review_coverage_roles_v TO ig_report_role;
CREATE PUBLIC SYNONYM review_coverage_roles_v FOR igops.review_coverage_roles_v;

GRANT SELECT ON igops.review_coverage_broles_v TO ig_report_role;
CREATE PUBLIC SYNONYM review_coverage_broles_v FOR igops.review_coverage_broles_v;